#### ######################################################### ####
#
# MAKEFILE (INCLUDE) FOR NOTESAPI C/C++ PROJECTS
# Author: Ulrich Krause
# Date:   2021/01/05
#
#### ######################################################### ####
#------------------------------------------------------------------
# COMMON
#------------------------------------------------------------------
    APIVER := 1101
    NOTESAPI = $(BASE_DIR)notesapi/$(APIVER)/

ifeq ($(TYPE),$(filter $(TYPE),$(strip application) $(strip addin)))
ifeq ($(OS),Windows_NT)
    TARGETEXT := .exe
    TARGET := $(TARGET)
else
    TARGETEXT :=
endif
else
ifeq ($(TYPE),$(strip shared))
ifeq ($(OS),Windows_NT)
    TARGETEXT := .dll
    TARGET := n$(TARGET)
else
    TARGETEXT := .so
    TARGET := lib$(TARGET)
endif    
else
ifeq ($(OS),Windows_NT)
    TARGETEXT := .lib
    TARGET := lib$(TARGET)
else
    TARGETEXT := .a
    TARGET := lib$(TARGET)
endif    
endif    #shared
endif # application

ifeq ($(OS),Windows_NT)
#==================================================================
#==================================================================
# WINDOWS
#==================================================================
#==================================================================
    DETECTED_OS := Windows
    PATHSEP=\\

    CC := cl.exe
    CXX := $(CC)
    LINK := link.exe
    LIBRARIAN := lib.exe

    MV = rename
    RM = del

    LIBPATH_PREFIX = /LIBPATH:
    INCLUDE_PREFIX = /I
    OUT = /OUT:
    OBJ = obj
    MMD =
    STD = /std:
    CPF := copy /Y 

    BOOTOBJ = notes0.obj notesai0.obj

ifeq ($(Platform),x86)
    PLATFORM=32
    NOTESLIBS := $(LIBPATH_PREFIX)$(NOTESAPI)lib/mswin32/
else
ifeq ($(Platform),x64)
    PLATFORM=64
    NOTESLIBS := $(LIBPATH_PREFIX)$(NOTESAPI)lib/mswin64/
endif
endif

    # -----------------------------------------------------------------
    # C O M M O N LIBS 
    # -----------------------------------------------------------------
    LIBS_COMMON := $(NOTESLIBS)   
    LIBS_COMMON += notes.lib 
    LIBS_COMMON += kernel32.lib 
    LIBS_COMMON += user32.lib 
    LIBS_COMMON += gdi32.lib 
    LIBS_COMMON += advapi32.lib 
    LIBS_COMMON += winspool.lib 

    # -----------------------------------------------------------------
    # C O M M O N PDB Settings
    # -----------------------------------------------------------------
    PDB_SETTINGS := /DEBUG 
    PDB_SETTINGS += /PDB:$(OUTDIR)$(TARGET).pdb 
    PDB_SETTINGS += /PDBSTRIPPED:$(OUTDIR)$(TARGET)_small.pdb 
    PDB_SETTINGS += /MAP:$(OUTDIR)$(TARGET).map

ifeq ($(BITNESS),32)
#------------------------------------------------------------------
# 32-BIT
#------------------------------------------------------------------
    # -----------------------------------------------------------------
    # C O M P I L E c/c++
    # -----------------------------------------------------------------
    CCFLAGS := /c
    CCFLAGS += /MT /D_MT
    CCFLAGS += /Ot /O2 /Ob2 /Oy-    
    CCFLAGS += /W4 /WX-
    CCFLAGS += /Gd /Gy /GF /Gs4096 /GS- /EHsc 
    CCFLAGS += /Zc:wchar_t- /Zi /Zl
    CCFLAGS += /DNT /DW32 /DW /D_X86_ /DND32
    CCFLAGS += /DDTRACE 
    CCFLAGS += /D_CRT_SECURE_NO_WARNINGS 
    CCFLAGS += /DPRODUCTION_VERSION 
    CCFLAGS += /DDUMMY
    CCFLAGS += /nologo
    CCFLAGS += /FI$(BASE_DIR)makefiles/prags.h
    # --------
    CXXFLAGS := $(CCFLAGS)
    # --------
    CXXFLAGS_STATIC := $(CCFLAGS)

    # -----------------------------------------------------------------
    # A P P L I C A T I O N
    # -----------------------------------------------------------------
    LDFLAGS_EXE := /MACHINE:X86 
    LDFLAGS_EXE += /SUBSYSTEM:CONSOLE 
    LDFLAGS_EXE += /INCREMENTAL:NO 
    LDFLAGS_EXE += $(PDB_SETTINGS)
    LDFLAGS_EXE += /NOLOGO 
    # --------
    LDLIBS_EXE := $(LIBS_COMMON)
    LDLIBS_EXE += ws2_32.lib 
    LDLIBS_EXE += msvcrt.lib 
    LDLIBS_EXE += vcruntime.lib 
    LDLIBS_EXE += ucrt.lib 
    LDLIBS_EXE += userenv.lib

    # -----------------------------------------------------------------
    # S H A R E D 
    # -----------------------------------------------------------------
    LDFLAGS_SHARED := /MACHINE:X86
    LDFLAGS_SHARED += /SUBSYSTEM:WINDOWS
    LDFLAGS_SHARED += /INCREMENTAL:NO 
    LDFLAGS_SHARED += $(PDB_SETTINGS)
    LDFLAGS_SHARED += /NOLOGO
    LDFLAGS_SHARED += /DLL
    LDFLAGS_SHARED += /OPT:REF

    # --------  
    LDLIBS_SHARED := $(LIBS_COMMON)
    LDLIBS_SHARED += comctl32.lib 
    LDLIBS_SHARED += comdlg32.lib

    # -----------------------------------------------------------------
    # S T A T I C
    # -----------------------------------------------------------------
    LDFLAGS_STATIC := /MACHINE:X86
    LDFLAGS_STATIC += /NOLOGO 
    # --------
    LDLIBS_STATIC := $(LIBS_COMMON)
    LDLIBS_STATIC += comctl32.lib 
    LDLIBS_STATIC += comdlg32.lib

else
#------------------------------------------------------------------
# 64-BIT
#------------------------------------------------------------------
    # -----------------------------------------------------------------
    # C O M P I L E c/c++
    # -----------------------------------------------------------------   
    CCFLAGS := /c
    CCFLAGS += /favor:INTEL64
    CCFLAGS += /MT /D_MT
    CCFLAGS += /Ot /O2 /Ob2 /Oy-    
    CCFLAGS += /W4 /WX-
    CCFLAGS += /Gd /Gy /GF /Gs4096 /GS- /EHsc 
    CCFLAGS += /Zc:wchar_t- /Zi /Zl
    CCFLAGS += /DNT /DW32 /DW /DW64 /DND64 /D_AMD64_ /DND64CLIENT /DND64SERVER /DDOMINO_AWS 
    CCFLAGS += /DHANDLE_IS_32BITS
    CCFLAGS += /DDTRACE 
    CCFLAGS += /D_CRT_SECURE_NO_WARNINGS 
    CCFLAGS += /DPRODUCTION_VERSION 
    CCFLAGS += /DDUMMY
    CCFLAGS += /nologo
    CCFLAGS += /FI$(BASE_DIR)makefiles/prags.h
    # --------    
    CXXFLAGS := $(CCFLAGS)
    # --------
    CXXFLAGS_STATIC := $(CCFLAGS)

    # -----------------------------------------------------------------
    # A P P L I C A T I O N
    # -----------------------------------------------------------------
    LDFLAGS_EXE := /MACHINE:X64 
    LDFLAGS_EXE += /SUBSYSTEM:CONSOLE 
    LDFLAGS_EXE += /INCREMENTAL:NO 
    LDFLAGS_EXE += /LARGEADDRESSAWARE
    LDFLAGS_EXE += $(PDB_SETTINGS)
    LDFLAGS_EXE += /NOLOGO 
    # --------
    LDLIBS_EXE := $(LIBS_COMMON)
    LDLIBS_EXE += ws2_32.lib 
    LDLIBS_EXE += msvcrt.lib 
    LDLIBS_EXE += vcruntime.lib 
    LDLIBS_EXE += ucrt.lib 
    LDLIBS_EXE += userenv.lib
 
    # -----------------------------------------------------------------
    # S H A R E D 
    # -----------------------------------------------------------------
    LDFLAGS_SHARED := /MACHINE:X64
    LDFLAGS_SHARED += /SUBSYSTEM:WINDOWS
    LDFLAGS_SHARED += /INCREMENTAL:NO 
    LDFLAGS_SHARED += /LARGEADDRESSAWARE
    LDFLAGS_SHARED += $(PDB_SETTINGS)
    LDFLAGS_SHARED += /NOLOGO  
    LDFLAGS_SHARED += /OPT:REF 
    LDFLAGS_SHARED += /DLL
     # --------
    LDLIBS_SHARED := $(LIBS_COMMON)
    LDLIBS_SHARED += msvcrt.lib 
    LDLIBS_SHARED += vcruntime.lib 
    LDLIBS_SHARED += ucrt.lib 
    LDLIBS_SHARED += comctl32.lib 
    LDLIBS_SHARED += comdlg32.lib

    # -----------------------------------------------------------------
    # S T A T I C
    # -----------------------------------------------------------------
    LDFLAGS_STATIC := /MACHINE:X64 
    LDFLAGS_STATIC += /NOLOGO 
    # --------
    LDLIBS_STATIC := $(LDLIBS_SHARED)

endif
else
#==================================================================
#==================================================================
# LINUX /UNIX
#==================================================================
#==================================================================
    DETECTED_OS := $(shell uname)  # same as "uname -s"
    PATHSEP=/

    CC := gcc
    CXX := g++
    LINK := g++
    LIBRARIAN := ar -rsc

    MV = mv -f
    RM = rm -f    

    LIBPATH_PREFIX = -L
    INCLUDE_PREFIX = -I
    OUT = -o 
    OBJ = o
    MMD = -MMD -o $@
    STD = -std=
    CPF := cp -f
ifeq ($(BITNESS),32)
#------------------------------------------------------------------
# 32-BIT  experimental as of 11.0.1
#------------------------------------------------------------------
    NOTESDIR = $(Notes_ExecDirectory32)
    CCFLAGS :=  -c -O -w -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -DUNIX -DLINUX -DLINUX86 -DW -DW32 -DDTRACE -fcheck-new -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC
    CXXFLAGS := -c -O -w -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -DUNIX -DLINUX -DLINUX86 -DW -DW32 -DDTRACE -fcheck-new -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC \
    -I/usr/local/include/g++ -I/usr/include/g++

    LDFLAGS_EXE := -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -felide-constructors -Wl,--whole-archive -rdynamic
    LDFLAGS_SHARED := -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -shared -fPIC -Wl,-Bsymbolic,--whole-archive,-ztext
    LDLIBS_EXE := -Wl,--no-whole-archive -L/$(NOTESAPI)lib/linux32 -lm -L/usr/local/lib -L/usr/lib -L$(NOTESDIR) -Wl,-rpath-link $(NOTESDIR) -lnotes -lpthread -lc -lresolv -ldl
    LDLIBS_SHARED := -Wl,--no-whole-archive -L/$(NOTESAPI)lib/linux32 -L/usr/local/lib -L/usr/lib -ldl -lrt -lm -lstdc++ -lpthread -lc -lresolv -lc

    LDFLAGS_STATIC :=
    LDLIBS_STATIC :=
    BOOTOBJ = $(NOTESAPI)lib/linux/notes0.o $(NOTESAPI)lib/linux/notesai0.o

else
#------------------------------------------------------------------
# 64-BIT
#------------------------------------------------------------------
    NOTESDIR = $(Notes_ExecDirectory)
    BOOTOBJ = $(NOTESAPI)lib/linux64/notes0.o $(NOTESAPI)lib/linux64/notesai0.o
    # -----------------------------------------------------------------
    # C O M P I L E c/c++
    # -----------------------------------------------------------------  
    CCFLAGS := -c -O -w -m64 -DGCC3 -DGCC4 
    CCFLAGS += -fno-strict-aliasing 
    CCFLAGS += -DGCC_LBLB_NOT_SUPPORTED 
    CCFLAGS += -Wformat 
    CCFLAGS += -Wall 
    CCFLAGS += -Wcast-align 
    CCFLAGS += -Wconversion 
    CCFLAGS += -DUNIX 
    CCFLAGS += -DLINUX 
    CCFLAGS += -DLINUX86 
    CCFLAGS += -DND64 
    CCFLAGS += -DW32 
    CCFLAGS += -DLINUX64 
    CCFLAGS += -DW 
    CCFLAGS += -DLINUX86_64 
    CCFLAGS += -DDTRACE 
    CCFLAGS += -DND64SERVER 
    CCFLAGS += -DDOMINO_AWS 
    CCFLAGS += -DPTHREAD_KERNEL 
    CCFLAGS += -D_REENTRANT 
    CCFLAGS += -DUSE_THREADSAFE_INTERFACES 
    CCFLAGS += -D_POSIX_THREAD_SAFE_FUNCTIONS  
    CCFLAGS += -DTHREADID_IS_64BITS 
    CCFLAGS += -DHANDLE_IS_32BITS 
    CCFLAGS += -DHAS_IOCP 
    CCFLAGS += -DHAS_BOOL 
    CCFLAGS += -DHAS_DLOPEN 
    CCFLAGS += -DUSE_PTHREAD_INTERFACES 
    CCFLAGS += -DLARGE64_FILES 
    CCFLAGS += -D_LARGEFILE_SOURCE 
    CCFLAGS += -D_LARGEFILE64_SOURCE 
    CCFLAGS += -DNDUNIX64 
    CCFLAGS += -DLONGIS64BIT 
    CCFLAGS += -DPRODUCTION_VERSION 
    CCFLAGS += -DOVERRIDEDEBUG 
    CCFLAGS += -fPIC 
    # --------
    CXXFLAGS := $(CCFLAGS)
    CXXFLAGS += -I/usr/local/include/g++ 
    CXXFLAGS += -I/usr/include/g++
    # --------
    CXXFLAGS_STATIC := $(CXXFLAGS)

    # -----------------------------------------------------------------
    # C O M M O N
    # -----------------------------------------------------------------
    FLAGS_COMMON := -m64 
    FLAGS_COMMON += -DGCC3 
    FLAGS_COMMON += -DGCC4 
    FLAGS_COMMON += -fno-strict-aliasing 
    FLAGS_COMMON += -DGCC_LBLB_NOT_SUPPORTED 
    FLAGS_COMMON += -Wformat 
    FLAGS_COMMON += -Wall 
    FLAGS_COMMON += -Wcast-align 
    FLAGS_COMMON += -Wconversion

    # -----------------------------------------------------------------
    # A P P L I C A T I O N
    # -----------------------------------------------------------------
    LDFLAGS_EXE := $(FLAGS_COMMON)
    LDFLAGS_EXE += -Wl,--whole-archive 
    LDFLAGS_EXE += -rdynamic 
    LDFLAGS_EXE += -felide-constructors
    # --------
    LDLIBS_EXE := -Wl,--no-whole-archive 
    LDLIBS_EXE += $(LIBPATH_PREFIX)/usr/local/lib64 
    LDLIBS_EXE += $(LIBPATH_PREFIX)/usr/lib64 
    LDLIBS_EXE += -lm 
    LDLIBS_EXE += -lpthread 
    LDLIBS_EXE += -lc 
    LDLIBS_EXE += -lresolv 
    LDLIBS_EXE += -ldl 
    LDLIBS_EXE += -lstdc++ 
    LDLIBS_EXE += -lnotes 
    LDLIBS_EXE += -lrt
    # -----------------------------------------------------------------
    # S H A R E D
    # -----------------------------------------------------------------
    LDFLAGS_SHARED := $(FLAGS_COMMON)
    LDFLAGS_SHARED += -Wl,-Bsymbolic,--whole-archive,-ztext 
    LDFLAGS_SHARED += -fPIC
    LDFLAGS_SHARED += -shared 
    # --------
    LDLIBS_SHARED := $(LDLIBS_EXE)
    # -----------------------------------------------------------------
    # S T A T I C
    # -----------------------------------------------------------------
    LDFLAGS_STATIC := $(FLAGS_COMMON)
    LDFLAGS_STATIC += -Wl,-Bsymbolic,--whole-archive,-ztext 
    LDFLAGS_STATIC += -fPIC
    LDFLAGS_STATIC += -static 
    # --------
    LDLIBS_STATIC := $(LDLIBS_EXE)
   
endif       # bitness
endif   # os

ifeq ($(TYPE),$(strip application))
    LDFLAGS = $(LDFLAGS_EXE)
    LDLIBS := $(LDLIBS_EXE)
else
ifeq ($(TYPE),$(strip addin))
    LDFLAGS = $(LDFLAGS_EXE)
    LDLIBS := $(BOOTOBJ) $(LDLIBS_EXE)
else
ifeq ($(TYPE),$(strip shared))
    LDFLAGS = $(LDFLAGS_SHARED)
    LDLIBS := $(LDLIBS_SHARED)    
else
    LINK = $(LIBRARIAN)
    CXXFLAGS = $(CXXFLAGS_STATIC)
    LDFLAGS = $(LDFLAGS_STATIC)
    LDLIBS := $(LDLIBS_STATIC)
endif    #shared
endif   #addin
endif  #application

OUTFILE = $(strip $(TARGET))$(strip $(TARGETEXT))
TARGET_EXT = $(TARGETEXT)
