#### ######################################################### ####
#
# MAKEFILE (INCLUDE) FOR NOTESAPI C/C++ PROJECTS
# Author: Ulrich Krause
# Date:   2021/01/05
#
#### ######################################################### ####
#------------------------------------------------------------------
# COMMON
#------------------------------------------------------------------
    APIVER := 901
    NOTESAPI = $(BASE_DIR)notesapi/$(APIVER)/

ifeq ($(TYPE),$(filter $(TYPE),$(strip application) $(strip addin)))
ifeq ($(OS),Windows_NT)
    TARGETEXT := .exe
    TARGET := $(TARGET)
else
    TARGETEXT :=
endif
else
ifeq ($(TYPE),$(strip shared))
ifeq ($(OS),Windows_NT)
    TARGETEXT := .dll
    TARGET := $(TARGET)
else
    TARGETEXT := .so
    TARGET := lib$(TARGET)
endif    
else
ifeq ($(OS),Windows_NT)
    TARGETEXT := .lib
    TARGET := $(TARGET)
else
    TARGETEXT := .a
    TARGET := lib$(TARGET)
endif    
endif    #shared
endif # application

ifeq ($(OS),Windows_NT)
#------------------------------------------------------------------
# WINDOWS
#------------------------------------------------------------------
    DETECTED_OS := Windows
    PATHSEP=\\

    CC := cl.exe
    CXX := cl.exe
    LINK := link.exe
    LIBRARIAN := lib.exe

    MV = rename
    RM = del

    LIBPATH_PREFIX = /LIBPATH:
    INCLUDE_PREFIX = /I
    OUT = /OUT:
    OBJ = obj
    MMD =
    STD = /std:
    CPF := copy /Y 

ifeq ($(Platform),x86)
    PLATFORM=32
else
ifeq ($(Platform),x64)
    PLATFORM=64
endif
endif

ifeq ($(BITNESS),32)
#------------------------------------------------------------------
# 32-BIT
#------------------------------------------------------------------
    CCFLAGS :=  -nologo -c -D_MT -MT /Zi /Ot /O2 /Ob2 /Oy- -Gd /Gy /GF /Gs4096 /GS- -EHsc /Zc:wchar_t- -Zl -W3 -DNT -DWIN32 -DW32 -DW -D_X86_ -DND32 -DDTRACE -D_CRT_SECURE_NO_WARNINGS -DPRODUCTION_VERSION  -DDUMMY
    CXXFLAGS := -nologo -c -D_MT -MT /Zi /Ot /O2 /Ob2 /Oy- -Gd /Gy /GF /Gs4096 /GS- -EHsc /Zc:wchar_t- -Zl -W3 -DNT -DWIN32 -DW32 -DW -D_X86_ -DND32 -DDTRACE -D_CRT_SECURE_NO_WARNINGS -DPRODUCTION_VERSION  -DDUMMY

    LDFLAGS_EXE := /DEBUG /MACHINE:X86 /SUBSYSTEM:CONSOLE /NOLOGO  /PDB:$(OUTDIR)$(TARGET).pdb /PDBSTRIPPED:$(OUTDIR)$(TARGET)_small.pdb \
    /LIBPATH:$(NOTESAPI)lib/mswin32/ 
    LDFLAGS_SHARED := -nologo -opt:ref /PDB:$(OUTDIR)$(TARGET).pdb /PDBSTRIPPED:$(OUTDIR)$(TARGET)_small.pdb -machine:x86 /SUBSYSTEM:WINDOWS -dll -entry:LibMain \
    /LIBPATH:$(NOTESAPI)lib/mswin32/ 
    
    LDLIBS_EXE := notes.lib msvcrt.lib kernel32.lib user32.lib gdi32.lib advapi32.lib winspool.lib ws2_32.lib userenv.lib
    LDLIBS_SHARED := notes.lib kernel32.lib user32.lib gdi32.lib advapi32.lib comctl32.lib winspool.lib comdlg32.lib
    
    CXXFLAGS_STATIC := /GS /W3 /Gy /Zc:wchar_t /Zi /Gm- /O2 /Zc:inline /fp:precise /D "WIN32" /D "NDEBUG" /D "W32" /WX- /Zc:forScope /Gd /Oy- /Oi /MT /FC /EHsc /nologo
    LDFLAGS_STATIC := /MACHINE:X86 /NOLOGO 
    LDLIBS_STATIC := /LIBPATH:$(NOTESAPI)lib/mswin32 notes.lib kernel32.lib user32.lib gdi32.lib advapi32.lib comctl32.lib winspool.lib comdlg32.lib

    BOOTOBJ = notes0.obj notesai0.obj
else
#------------------------------------------------------------------
# 64-BIT
#------------------------------------------------------------------
    CCFLAGS :=  /favor:INTEL64 -nologo -c -D_MT -MT /Zi /Ot /O2 /Ob2 /Oy- -Gd /Gy /GF /Gs4096 /GS- /EHsc /Zc:wchar_t- -Zl -W3 -DNT -DWIN32 -DW32 -DW -DW64 -DND64 -D_AMD64_ -DDTRACE -D_CRT_SECURE_NO_WARNINGS -DND64SERVER -DPRODUCTION_VERSION  -DDUMMY
    CXXFLAGS := /favor:INTEL64 -nologo -c -D_MT -MT /Zi /Ot /O2 /Ob2 /Oy- -Gd /Gy /GF /Gs4096 /GS- /EHsc /Zc:wchar_t- -Zl -W3 -DNT -DWIN32 -DW32 -DW -DW64 -DND64 -D_AMD64_ -DDTRACE -D_CRT_SECURE_NO_WARNINGS -DND64SERVER -DPRODUCTION_VERSION  -DDUMMY
    
    LDFLAGS_EXE := /DEBUG /MACHINE:X64 /SUBSYSTEM:CONSOLE /NOLOGO /PDB:$(OUTDIR)$(TARGET).pdb /PDBSTRIPPED:$(OUTDIR)$(TARGET)_small.pdb /LARGEADDRESSAWARE
    LDLIBS_EXE := /LIBPATH:$(NOTESAPI)lib/mswin64 notes.lib msvcrt.lib kernel32.lib user32.lib gdi32.lib advapi32.lib winspool.lib ws2_32.lib userenv.lib
 
    LDFLAGS_SHARED := /DEBUG /MACHINE:X64 /SUBSYSTEM:CONSOLE /NOLOGO /PDB:$(OUTDIR)$(TARGET).pdb /PDBSTRIPPED:$(OUTDIR)$(TARGET)_small.pdb /LARGEADDRESSAWARE /DLL   
    LDLIBS_SHARED := /LIBPATH:$(NOTESAPI)lib/mswin64 notes.lib kernel32.lib user32.lib gdi32.lib advapi32.lib comctl32.lib winspool.lib comdlg32.lib

    CXXFLAGS_STATIC := /favor:INTEL64 /GS /W3 /Gy /Zc:wchar_t /Zi /Gm- /O2 /Ob2 /Zc:inline /fp:precise /D "NDEBUG" /D "NT" /D "WIN32" /D "_WINDOWS" /D "W64" /D "HANDLE_IS_32BITS" /D "ND64" /D "_MBCS" /WX- /Zc:forScope /Gr /Oi /MT /FC /EHsc /nologo
    LDFLAGS_STATIC := /MACHINE:X64 /NOLOGO 
    LDLIBS_STATIC := /LIBPATH:$(NOTESAPI)lib/mswin64 notes.lib kernel32.lib user32.lib gdi32.lib advapi32.lib comctl32.lib winspool.lib comdlg32.lib

    BOOTOBJ = notes0.obj notesai0.obj
endif
else
#------------------------------------------------------------------
# LINUX /UNIX
#------------------------------------------------------------------
    DETECTED_OS := $(shell uname)  # same as "uname -s"
    PATHSEP=/

    CC := gcc
    CXX := g++
    LINK := g++
    LIBRARIAN := ar -rsc

    MV = mv -f
    RM = rm -f    

    LIBPATH_PREFIX = -L
    INCLUDE_PREFIX = -I
    OUT = -o 
    OBJ = o
    MMD = -MMD -o $@
    STD = -std=
    CPF := cp -f
ifeq ($(BITNESS),32)
#------------------------------------------------------------------
# 32-BIT
#------------------------------------------------------------------
    NOTESDIR = $(Notes_ExecDirectory32)
    CCFLAGS :=  -c -O -w -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -DUNIX -DLINUX -DLINUX86 -DW -DW32 -DDTRACE -fcheck-new -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC
    CXXFLAGS := -c -O -w -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -DUNIX -DLINUX -DLINUX86 -DW -DW32 -DDTRACE -fcheck-new -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC \
    -I/usr/local/include/g++ -I/usr/include/g++

    LDFLAGS_EXE := -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -felide-constructors -Wl,--whole-archive -rdynamic
    LDFLAGS_SHARED := -m32 -march=pentium3 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -shared -fPIC -Wl,-Bsymbolic,--whole-archive,-ztext
    LDLIBS_EXE := -Wl,--no-whole-archive -L/$(NOTESAPI)lib/linux32 -lm -L/usr/local/lib -L/usr/lib -L$(NOTESDIR) -Wl,-rpath-link $(NOTESDIR) -lnotes -lpthread -lc -lresolv -ldl
    LDLIBS_SHARED := -Wl,--no-whole-archive -L/$(NOTESAPI)lib/linux32 -L/usr/local/lib -L/usr/lib -ldl -lrt -lm -lstdc++ -lpthread -lc -lresolv -lc

    LDFLAGS_STATIC :=
    LDLIBS_STATIC :=
    BOOTOBJ = $(NOTESAPI)lib/linux/notes0.o $(NOTESAPI)lib/linux/notesai0.o

else
#------------------------------------------------------------------
# 64-BIT
#------------------------------------------------------------------
    NOTESDIR = $(Notes_ExecDirectory)
    CCFLAGS :=  -c -O -w -m64 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion -DUNIX -DLINUX -DLINUX86 -DND64 -DW32 -DLINUX64 -DW -DLINUX86_64 -DDTRACE -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS  -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DNDUNIX64 -DLONGIS64BIT -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC 
    CXXFLAGS := -c -O -w -m64 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion -DUNIX -DLINUX -DLINUX86 -DND64 -DW32 -DLINUX64 -DW -DLINUX86_64 -DDTRACE -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS  -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DNDUNIX64 -DLONGIS64BIT -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC \
    -I/usr/local/include/g++ -I/usr/include/g++
    CXXFLAGS_STATIC := -c -O -w -m64 -DNT -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion -DUNIX -DLINUX -DLINUX86 -DND64 -DW32 -DLINUX64 -DW -DLINUX86_64 -DDTRACE -DPTHREAD_KERNEL -D_REENTRANT -DUSE_THREADSAFE_INTERFACES -D_POSIX_THREAD_SAFE_FUNCTIONS  -DHANDLE_IS_32BITS -DHAS_IOCP -DHAS_BOOL -DHAS_DLOPEN -DUSE_PTHREAD_INTERFACES -DLARGE64_FILES -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -DNDUNIX64 -DLONGIS64BIT -DPRODUCTION_VERSION -DOVERRIDEDEBUG -fPIC \
    -I/usr/local/include/g++ -I/usr/include/g++    
    
    LDFLAGS_EXE := -m64 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion  -felide-constructors -Wl,--whole-archive -rdynamic
    LDLIBS_EXE := -Wl,--no-whole-archive -L/usr/local/lib64 -L/usr/lib64 -lstdc++ -L$(NOTESDIR) -Wl,-rpath-link $(NOTESDIR) -lnotes -ldl -lrt -lm -lpthread -lc -lresolv -lc
    
    LDFLAGS_SHARED := -m64 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion -shared -fPIC -Wl,-Bsymbolic,--whole-archive,-ztext
    LDLIBS_SHARED := -Wl,--no-whole-archive -L/usr/local/lib64 -L/usr/lib64 -ldl -lrt -lm -lstdc++ -lpthread -lc -lresolv -lc

    LDFLAGS_STATIC := -m64 -DGCC3 -DGCC4 -fno-strict-aliasing -DGCC_LBLB_NOT_SUPPORTED -Wformat -Wall -Wcast-align -Wconversion -static -fPIC -Wl,-Bsymbolic,--whole-archive,-ztext
    LDLIBS_STATIC := -Wl,--no-whole-archive -L/usr/local/lib64 -L/usr/lib64 -lstdc++ -lnotes -ldl -lrt -lm -lpthread -lc -lresolv -lc
    
    BOOTOBJ = $(NOTESAPI)lib/linux64/notes0.o $(NOTESAPI)lib/linux64/notesai0.o
endif       # bitness
endif   # os

ifeq ($(TYPE),$(strip application))
    LDFLAGS = $(LDFLAGS_EXE)
    LDLIBS := $(LDLIBS_EXE)
else
ifeq ($(TYPE),$(strip addin))
    LDFLAGS = $(LDFLAGS_EXE)
    LDLIBS := $(BOOTOBJ) $(LDLIBS_EXE)
else
ifeq ($(TYPE),$(strip shared))
    LDFLAGS = $(LDFLAGS_SHARED)
    LDLIBS := $(LDLIBS_SHARED)    
else
    LINK = $(LIBRARIAN)
    CXXFLAGS = $(CXXFLAGS_STATIC)
    LDFLAGS = $(LDFLAGS_STATIC)
    LDLIBS := $(LDLIBS_STATIC)
endif    #shared
endif   #addin
endif  #application

OUTFILE = $(strip $(TARGET))$(strip $(TARGETEXT))
TARGET_EXT = $(TARGETEXT)
