#### ######################################################### ####
# MAKEFILE (INCLUDE) FOR C/C++ PROJECTS
# Author: Ulrich Krause
# Date:   2021/01/05
#### ######################################################### ####
#------------------------------------------------------------------
# COLOR CODES
#------------------------------------------------------------------
reset = [0m
green = [32m
yellow =[33m
GREEN = [92m
RED = [91m

ifeq ($(SHOWFLAGS),1)
ECHO_OR_DEBUG =
else 
ECHO_OR_DEBUG =-@
endif

INCLUDES += $(NOTESAPI)include
#------------------------------------------------------------------
# RECIPES
#------------------------------------------------------------------
make: start_msg $(TARGET)$(TARGET_EXT)
$(TARGET)$(TARGET_EXT): $(OBJS)
	-@ echo $(reset)
	-@ echo $(yellow)-- Build $(OUTFILE):$(reset)
	$(ECHO_OR_DEBUG) $(strip $(LINK) $(OUT)$(OUTFILE) $(LDFLAGS) *.$(OBJ) \
	$(addprefix $(LIBPATH_PREFIX),$(LDLIBS_ADDITIONAL_PATH)) \
	$(LDLIBS) $(MDF))
	-@ echo $(OUTFILE) built.

ifeq ($(DETECTED_OS),$(strip Windows))
ifneq ($(TYPE),$(strip static))
ifneq ($(wildcard $(TARGET).pdb),)
	-@ $(RM) $(TARGET).pdb $(TARGET).sym $(TARGET).ilk
	-@ $(MV) $(TARGET)_small.pdb $(TARGET).pdb
endif
	-@ echo $(reset)
	-@ echo $(yellow)-- Build $(TARGET).pdb:$(reset)
	-@ echo $(TARGET).pdb built.
endif
endif

$(OBJS): %.$(OBJ): %.$(SUFFIX)
ifeq ($(SUFFIX),cpp)
	$(ECHO_OR_DEBUG) $(strip $(CXX) \
	$(WINVER) $(addprefix $(STD),$(LANGUAGE)) \
	$(CXXFLAGS) $(addprefix $(INCLUDE_PREFIX),$(INCLUDES)) -c $<)
else
	$(ECHO_OR_DEBUG) $(strip $(CC) \
	$(WINVER) $(addprefix $(STD),$(LANGUAGE)) \
	$(CCFLAGS) $(addprefix $(INCLUDE_PREFIX),$(INCLUDES)) -c $<)
endif

.PHONY: start_msg
start_msg:
	-@ echo $(reset)
	-@ echo $(GREEN)-- Build started: Project: $(TARGET), $(DETECTED_OS)/$(BITNESS)bit
	-@ echo $(yellow)-- Compile sources:$(reset)

.PHONY: install
install:
ifneq ($(INSTALLDIR),)
	-@ echo $(reset)
	-@ echo $(GREEN)-- Installing: $(reset)
	$(CPF) $(TARGET)$(TARGETEXT) $(DESTDIR)$(INSTALLDIR)
ifneq ($(wildcard $(TARGET).pdb),)
	$(CPF) $(TARGET).pdb $(DESTDIR)$(INSTALLDIR)
endif
else
	-@ echo $(reset)
	-@ echo $(RED)No INSTALLDIR provided. Aborting.$(reset)
endif

.PHONY: uninstall
uninstall:
ifneq ($(INSTALLDIR),)
	-@ echo $(reset)
	-@ echo $(GREEN)-- Uninstalling: $(reset)
ifneq ($(wildcard $(DESTDIR)$(INSTALLDIR)$(OUTFILE)),)
	$(RM) $(DESTDIR)$(INSTALLDIR)$(OUTFILE)
endif
ifneq ($(wildcard $(DESTDIR)$(INSTALLDIR)$(TARGET).pdb),)
	$(RM) $(DESTDIR)$(INSTALLDIR)$(TARGET).pdb
endif
else
	-@ echo $(reset)
	-@ echo $(RED)No INSTALLDIR provided. Aborting.$(reset)
endif

.PHONY: info
info: 
	-@ echo $(reset)
	-@ echo $(GREEN)-- Info: $(reset)
	-@ echo Target: $(TARGET)$(TARGETEXT)
	-@ echo Include paths: $(INCLUDES)
	-@ echo Source paths: $(SRCDIR)
	-@ echo Source files: $(SRCS)
	-@ echo Object files: $(OBJS)

.PHONY: clean
clean:
	-@ echo $(reset)
	-@ echo $(GREEN)-- Clean:$(reset)
	-@ $(RM) *.$(OBJ)
